#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

# include <gtk/gtk.h>
# include <glade/glade.h>

#include "mysqlstuff.h" /* Currently assumes a MYSQL database */

#define GEQVERSION "1.17"
#define GEQUELVERSION " v" GEQVERSION ", Aug. 2008 "

#define LIBGLADEXMLFILE "gequel.glade-3"

#define DEBUG /* MACRO to make debugging easier */
#define DEBUGLEVEL 1 /* only for ldebug calls */

#define TABCOUNT   4 /* number of tabs in notebook */
#define STYLECOUNT 4 /* number of styles we want to reserve (colors & fonts) */

enum Tabs   { RECORDSTAB , FIELDSTAB, SEARCHTAB, QUERYTAB };
enum Styles { STLDEFAULT , STLRED, STLBOLD, STLBLDRED };

#define DEFFONT  "-*-helvetica-medium-r-normal--12-*-*-*-*-*-*-*"
#define BOLDFONT "-*-helvetica-bold-r-normal--12-*-*-*-*-*-*-*"

#define FONTWIDTH  7 /* reserved space for one character in browser column */

#define SELECT  "SELECT * FROM %s "
#define INSERT  "INSERT INTO %s VALUES ("
#define UPDATE  "UPDATE %s SET "
#define DELETE  "DELETE FROM %s "
#define WHERE   " WHERE "
#define EQUAL   " = "
#define LIKE    " LIKE "
#define AND     " AND "
#define ORDERBY " ORDER BY "

#define PASSIVE  0
#define ACTIVE   1
#define KEYFIELD 2

#define COUNT    0
#define NOCOUNT  1

#define MAXLOGSIZE 50000
#define MAXQUERY   10000
#define MAXMESSAGE 1024

#define DELIMITER ","         /* field delimiter for ASCII output. Here set to comma */
/* #define DELIMITER "\t" */  /* field delimiter for ASCII output. Here set to <TAB> */

/* contains info for one single record */
typedef struct _Record
{
   char      *table;       /* currently selected table */
   int       numflds;      /* number of fields (without opt. indices) */
   int       sortkey;      /* number of field we'll use to sort */
   SQL       *dbsock;      /* MYSQL style : int dbsock for MSQL */
   SQL_FIELD *fields;      /* stores fields for current table */
   GtkStyle  *styles[STYLECOUNT];
   
   /* until here we overlap with the Tabinfo struct : consider using union ? */

   char      *selectedname; /* hook for fileselector to store choice */

   char      **contents;   /* field contents as obtained from database */
   GtkWindow *window;      /* top window for this record */
   GtkWidget *gtable;      /* holds info: do not confuse with dbase table ! */
   GtkEntry  **entries;    /* (changed) field contents inside GUI textentry */

   void      *tabinfohook; /* place to hookup Tabinfo struct (unused) */
} Record;

/* contains info for one single browser panel (tab) */
typedef struct _TabInfo
{
   char      *table;       /* currently selected table */
   int       numflds;      /* number of dbfields (without opt. indices) */
   int       sortkey;      /* number of field we'll use to sort */
   SQL       *dbsock;      /* MYSQL style : int dbsock for MSQL */
   SQL_FIELD *fields;      /* stores fields for current table */
   GtkStyle  *styles[STYLECOUNT];
   
   GtkWindow *mainwindow;

   /* until here we overlap with the Params struct : consider using union ? */
   
   int       nrcols;       /* number of columns (often fields) in browser */
   int       nrrows;       /* number of rows (records) in browser */
   
   GtkWidget *clist;       /* pointer to clist widget in notebook */
   GtkWidget *frame;       /* frame serving as container for clist */
   
   gchar    **titles;
   int      selectedrow;   /* number of selected row (or -1 if none) */
   char     *selectedname; /* hook for fileselector to store choice */
    
   void     *paramshook;   /* place to hookup Global params struct (unused) */
} TabInfo;

/* contains all global application info */
typedef struct _Params
{
   char      *table;       /* currently selected table */
   int       numflds;      /* number of fields (only for counter) */
   int       sortkey;      /* number of field we'll use to sort */
   SQL       *dbsock;      /* MYSQL style : int dbsock for MSQL */
   SQL_FIELD *fields;      /* stores fields for current table */
   GtkStyle  *styles[STYLECOUNT];

   GtkWindow *mainwindow;
   GtkWidget *querywindow;
   GtkWidget *logwindow;   /* widget main window: need it for log messages */
   GtkWidget *notebook;

   /* Gtk2 embeds text buffer widgets inside a GtkTextView widget, while Gtk1.2
    uses a single GtkText widget for both parts */
  
   GtkTextBuffer *querybuffer;
   GtkTextBuffer *logbuffer; 
   
   int       numrecs;      /* number of records (only for counter) */

   char      *user;
   char      *pwd;
   char      *server;
   char      *database;
   char      *selectedname; /* hook for fileselector to store choice */
   int       countflag;     /* count records when loading new table ? */
   
   int       nrtabs;        /* number of tabpages in browser notebook */
   TabInfo   *tabs[TABCOUNT];
   
   GtkOptionMenu *tablelist;
   GtkOptionMenu *dbaselist;

} Params;

/* main.c : */
Params *getParams();     /* returns the Params structure from main */
GtkWindow *get_top_window(); /* returns the top_window id as stored in main */
GtkWindow* create_mainwindow (void);
GtkWindow* create_recordwindow (void);
GtkWindow* create_dialogbox (void);

/* startup.c */
void gequelInit(Params *p, int argc, char *argv[]);
void storeWidgets(GtkWindow *main, Params *p);
void connectMainWindowSignals(GtkWindow *main, Params *p);

/* dialog.c : */
void ldebug(int level, char *format, ...);
void defaultAction(GtkButton *button, gpointer dummy);
void showMessageBox(char *s);
void showConfirmBox(char *s, gpointer yes, gpointer no, gpointer hook);
void showVarArgMessage(char *s,...);
void showMessage(char *s);
void setLogWindow(Params *p);
void showSqlError(SQL *sock, char *s,...);
void writeLog(char *s,...);
void saveLogAction(GtkWidget *w, gpointer user_data );

/* helpers.c : */
void setStringValue(GtkWindow *window, char *widgetname,char *string);
int  executeQuery(SQL *sock, char *query);
int  getNumRecords(Params *p);
void setGUICounters(Params *p, int nrFields, int nrRecords);
void setSortKey(int row);
void newTab(TabInfo *ti, gpointer callback, char *refname);
void destroyTab(TabInfo *ti);
void destroyAllTabs(Params *p);
void refreshRecordTab();
void refreshFieldTab();
GtkStyle *newFontStyle(GtkStyle *source, gchar *fontname);
GtkStyle *newColorStyle(GtkStyle *source, double rgb[3]);
GtkStyle *newColorFontStyle(GtkStyle *source, double rgb[3], gchar *fontname);

/* sqlconnect.c : */
void setNewServer(Params *p, char *server);
void setDBaseList(Params *p, char *preselect);
void setTableList(Params *p, char *preselect);
void setNewDatabase(Params *p, char *database);
void setNewTable(Params *p, char *table);

/* optionactions.c : */
void on_database_activate(GtkMenuItem *item, gpointer  user_data);
void on_table_activate(GtkMenuItem *item, gpointer  user_data);

/* showrecords.c : */
void showRecords(Params *p);
void setTabColumnWidths(TabInfo *ti, int type); /* now generic: to helpers.c ? */
void saveTabAction(GtkWidget *w, gpointer user_data); /* now generic: to helpers.c ? */
  
/* showfields.c : */
int  getSortKey(Params *p);
void showFields(Params *p);

/* showquery.c : */
void showQuery(Params *p, char *query);
void loadQueryAction(GtkWidget *w, gpointer user_data );
void saveQueryAction(GtkWidget *w, gpointer user_data );

/* showsearch.c : */
void showSearch(Params *p, Record *rec);

/* recordform.c : */
void on_recorddelete_clicked(GtkButton *dummy, gpointer rec); /* callback */
void destroyRecordWindow(Record *rec);
void newModifyWindow(TabInfo *ti);
void newCopyRecordWindow(Record *rec);
void newCopyMainWindow(TabInfo *ti);
void newInsertWindow(Params *p);
void newSearchWindow(Params *p);
void saveRecordAction(GtkWidget *w, gpointer user_data);

/* sqlactions.c : */
char *addStringQuotes(int type);
void updateRecordAction(GtkButton *button, gpointer rec);
void deleteRecordAction(GtkButton *button, gpointer rec);
void insertRecordAction(GtkButton *button, gpointer rec);

/* structhandlers.c : */
TabInfo   *newTabInfostruct(Params *p, int tabid, int nrcols, int nrrows);
void      destroyTabInfostruct(TabInfo *ti);
Record    *newRecordstruct(TabInfo *ti);
Record    *copyRecordstruct(Record *rec);
void      destroyRecordstruct(Record *);

/* filestuff.c : */
void newFileSelector(char *preset, char **result, GtkFileChooserAction action);

/* libglade.c : */
void add_to_search_path (const gchar *dir);
GtkWidget *lookup_widget (GtkWindow *root, char *widgetname);
GtkWidget* create_widget (char *widgetname);  
